@@ -1,3 +1,19 @@
+"""
+This file is part of Candela.
+
+Candela is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
+
+Candela is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with Candela.  If not, see <http://www.gnu.org/licenses/>.
+"""
 import constants
 
 
@@ -50,11 +66,11 @@ class Command(object):
                 if reqd:
                     required_params.append(kw)
             if len(args) + len(kwargs) < len(required_params):
-                return (False, "Missing arguments")
+                return (False, "Usage: %s" % self.definition)
             for kw in self.kwargs.keys():
                 name,reqd = self.kwargs[kw]
                 if reqd and kw not in kwargs.keys():
-                    return (False, "Missing argument %s (-%s) - %s" % (name, kw, kwargs))
+                    return (False, "Usage: %s" % (self.definition))
             return (True, "")
         self.validate = validator
         self.default_validate = validator
