@@ -1,5 +1,22 @@
+"""
+This file is part of Candela.
+
+Candela is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
+
+Candela is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with Candela.  If not, see <http://www.gnu.org/licenses/>.
+"""
 import curses
 import sys
+import signal
 import threading
 import textwrap
 import platform
@@ -24,6 +41,8 @@ class Shell():
         scriptfile - the name of the script file to run. If not None and the
                      file exists, the script will be immediately run.
         """
+        self._register_sigint_handler()
+
         self.script_lines = self._parse_script_file(scriptfile)
         self.script_counter = 0
         self.scriptfile = ""
@@ -44,6 +63,13 @@ class Shell():
 
         # should the command menu be shown
         self.should_show_help = True
+        # for commands with only positional args, show the
+        # name of the next argument as the user types
+        self.should_show_hint = False
+
+        # dictionary of functions to call on key events
+        # keys are chars representing the pressed keys
+        self.keyevent_hooks = {}
 
         # the text to stick in the upper left corner of the window
         self.header = ""
@@ -272,6 +298,13 @@ class Shell():
             _y,_x = self.stdscr.getyx()
             index = _x - len(self.prompt)
             #self.stdscr.addstr(20, 70, str(keyin))  # for debugging
+            try:
+                if chr(keyin) in self.keyevent_hooks.keys():
+                    cont = self.keyevent_hooks[chr(keyin)](chr(keyin), buff)
+                    if cont == False:
+                        continue
+            except:
+                pass
             if keyin in [127, 263]:  # backspaces
                 del_lo, del_hi = self._get_backspace_indices()
                 buff = buff[:index+del_lo] + buff[index+del_hi:]
@@ -294,7 +327,7 @@ class Shell():
                     if len(buff.split()) == 1 and not buff.endswith(' '):
                         buff = choices[0]
                     else:
-                        if len(buff.split()) != 1:
+                        if len(buff.split()) != 1 and not buff.endswith(' '):
                             buff = ' '.join(buff.split()[:-1])
                         if buff.endswith(' '):
                             buff += choices[0]
@@ -309,6 +342,15 @@ class Shell():
                 buff = buff[:index-1] + chr(keyin) + buff[index-1:]
                 self._redraw_buffer(buff)
                 self.stdscr.move(_y, min(_x, len(buff) + len(self.prompt)))
+                if self.should_show_hint and keyin == 32:
+                    command = self._get_command(buff)
+                    if hasattr(command, 'definition') and '-' not in command.definition:
+                        try:
+                            nextarg = command.definition.split()[len(buff.split())]
+                            self.stdscr.addstr(_y, _x+1, nextarg)
+                            self.stdscr.move(_y, _x)
+                        except:
+                            pass
         self.put(buff, command=True)
         self.stdscr.refresh()
         return buff
@@ -365,8 +407,11 @@ class Shell():
         commands = []
         if menu:
             commands = menu.commands
+        if len(commands) == 0:
+            self.put("No commands found. Maybe you forgot to set self.menus or self.menu?")
+            self.put("Hint: use F1 to quit")
         for command in commands:
-            if command.name == buff.split()[0]:
+            if command.name == buff.split()[0] or buff.split()[0] in command.aliases:
                 return command
         return None
 
@@ -445,32 +490,24 @@ class Shell():
             if len(tokens) == 0:
                 self.put("\n")
                 continue
-            menu = self.get_menu()
-            commands = []
-            if menu:
-                commands = menu.commands
-            for command in commands:
-                if tokens[0] == command.name or tokens[0] in command.aliases:
-                    try:
-                        args, kwargs = command.parse_command(tokens)
-                    except Exception as e:
-                        self.put(e)
-                        break
-                    success, message = command.validate(*args, **kwargs)
-                    if not success:
-                        self.put(message)
-                        break
+            command = self._get_command(choice)
+            if not command:
+                self.put("Invalid command - no match")
+                continue
+            try:
+                args, kwargs = command.parse_command(tokens)
+                success, message = command.validate(*args, **kwargs)
+                if not success:
+                    self.put(message)
+                else:
+                    ret_choice = command.run(*args, **kwargs)
+                    if ret_choice == constants.CHOICE_INVALID:
+                        self.put("Invalid command")
                     else:
-                        ret_choice = command.run(*args, **kwargs)
                         if command.new_menu and ret_choice != constants.FAILURE:
                             self.menu = command.new_menu
-            if success:
-                if ret_choice == constants.CHOICE_INVALID:
-                    self.put("Invalid command")
-                if len(commands) == 0:
-                    self.put("No commands found. Maybe you forgot to set self.menus or self.menu?")
-                    self.put("Hint: use F1 to quit")
-
+            except Exception as e:
+                self.put(e)
         return self
 
     def get_menu(self):
@@ -543,6 +580,16 @@ class Shell():
         """
         curses.endwin()
 
+    def _register_sigint_handler(self):
+        """
+        Properly handle ^C and any other method of sending SIGINT.
+        This avoids leaving the user with a borked up terminal.
+        """
+        def signal_handler(signal, frame):
+            self.end()
+            sys.exit(0)
+        signal.signal(signal.SIGINT, signal_handler)
+
     def _update_screen(self):
         """
         Refresh the screen and redraw all elements in their appropriate positions
